# todo

# Live trading

# 1. Make an apscheduler thing that can pass jobs to ibinsync
# 2. Make a way to configure which algos are run when - this should be loaded via config file initially.
# 3. Run those algos at those times.

# IB should be pluggable (i.e. we could use something else)
